# Run AWS Greengrass on DE10-Nano Development Kit device running Ubuntu16.04

# Table of Contents

- [Introduction](#Introduction)
- [Step 1: Prerequisites](#Prerequisites)
- [Step 2: Prepare Your Device](#PrepareDevice)
- [Step 3: Setup Greengrass Core](#InstallGreengrass)
- [Next Steps](#NextSteps)
- [References](#References)


<a name="Introduction"></a>
# Introduction

This document describes how to run AWS Greengrass on DE10-Nano Development Kit device running Ubuntu 16.04. It covers the installation and execution of greengrass core in several steps. This multi-step process includes:
-   Preparing DE10-Nano device
-   Installing AWS Greengrass Core
-   Enabling Greengrass as a daemon process

# Step 1: Prerequisites

You should have the following items ready:

**DE10-Nano Development Kit**

- You can purchase it from Terasic who is the developer company of the device [here][LINK_Terasic_DE10-Nano-Purchase].
- DE10-Nano Development Kit has following items:
  - **DE10-Nano device**
  - **8GB MicroSD card**
  - **Ethernet cable**
  - **USB cable (Type-A to Mini-B)**

**Host PC and Display**

- Ubuntu 16.04 is installed on the host PC as the developing environment. Hence commands listed in this article are for Ubuntu environment. If you want to use OS such like Windows or other Linux distribution, please alternate reading depending on your OS.

## Optional:

**MicroSD Card USB Reader**

- If you don't have MicroSD card slot on your host PC, this item is necessary. 

**HDMI Cable and Display with HDMI Input**

- Since DE10-Nano has HDMI output, it can boot up with GUI like Raspberry Pi. In fact, we will boot GUI-enabled Ubuntu 16.04 linux on DE10-Nano. So if you want to use GUI, HDMI cable and another display are necessary.
- If you want to display GUI, you need to have following items, as DE10-Nano doesn't come with USB Type-A adapter and has **only one USB Micro-A port**. If you want to use USB keyboard and mouse, you need to get an USB adapter and USB Hub.
  - **USB adapter (male Micro-A to female USB Type-A)**
  - **USB Hub**

Note:  
**USB adapter (male Micro-A to female USB Type-A)** is different from **Male Micro-B**.  
It's very easy to make mistake because male Micro-B is is more well-known than Micro-A in general. 

# Step 2: Prepare Your Device

## Download the bootable image

At first, you need to install a bootable image on your SD card.  
The bootable image is available to download [here][Bootable-image-link].  
The image contains Lubuntu 16.04 image.

As the bootable image is compressed with zip, you need to unzip it.

```
cd ~/Downloads
unxz DE10-Nano-Cloud-Native.zip
```

**Note:** This tutorial describes how to install the image in Linux environment. If you use another OS such as Windows OS, please find the way to install microSD card image yourself.

## Write the image to the microSD card

Insert the microSD card to your PC and identify its path with dmesg or lsblk command, assuming its device path exists in /dev/sdb.
Using dd command to write the image to your microSD card.  
**Please make sure the device path is correct.**  
**dd command writes the image specified by if="image" to the device path specified by of="device path".**  
**If you choose another device path as the destination of dd, it may crash your PC.**  
With the option of "status=progress", you can check the progress of dd command.

```
sudo dd if=DE10-Nano-Cloud-Native of=/dev/sdb status=progress
```

## Setup MSEL on DE10-Nano

DE10-Nano has some switches named MSEL to configure the board setting. You need to setup the MSEL swtiches to run Linux on the device.
According to Chapter 2.2 MSEL Settings of [this document][terasic-getting-started-guide], setup MSEL[4:0] to 01010.

![MSEL-settings](de10-nano-linux-c.md.pic/MSEL-settings.png)

## Boot DE10-Nano

There are three options supported to operate the device.
They are "Using GUI", "Using Serial Console", and "Using SSH".

### Using Serial Console

Install serial console application like PuTTY or minicom on your PC.

Connect UART-TO-USB port shown in the picture below and USB port of your PC with USB cable(Type-A to Mini-B).

![UART-TO-USB](de10-nano-linux-c.md.pic/UART-TO-USB.png)

Type dmesg to identify what device path is.

```
dmesg | tail

[12782.413742] usb 1-2.4: Detected FT232RL
[12782.414335] usb 1-2.4: FTDI USB Serial Device converter now attached to ttyUSB0
```

where ttyUSB0 is given for the USB cable.

Open PuTTY and setup the parameters for serial connection.
**Note: sudo is necessary**  
Configuration for serial connection are below.

- 115200 baud rate
- no parity
- 1 stop bit
- no flow control settings

If the device path and baud rate are set as shown in the figure below, there should be no problem.

![PuTTY-serial-settings](de10-nano-linux-c.md.pic/PuTTY-serial-settings.png)


Click "Open" and if you see a black box window, push enter key.
If you see an interactive login shell, you can login.

![PuTTY-blackbox](de10-nano-linux-c.md.pic/PuTTY-Blackbox.png)

Username and Password are here.
**UserName: "root", Password: "de10nano"**

### Using SSH

If you have a router or modem which connects your PC and DE10-Nano are on the same network, using SSH would be a good choice.

Before you boot DE10-Nano, connect DE10-Nano to your moderm or router with an ethernet cable.

Boot the device and wait for about a minute.
During the boot process, ssh daemon is called and SSH login is enabled.

At the beginning you need to identify your network address by typing "ip addr" or "ifconfig".

```
ip addr

1: lo:
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
    inet 127.0.0.1/8 scope host lo
       valid_lft forever preferred_lft forever
2: eno1:
    link/ether 94:c6:91:a0:76:17 brd ff:ff:ff:ff:ff:ff
    inet 192.168.3.28/24 brd 192.168.3.255 scope global dynamic eno1
```

We can see the network address is "192.168.3.XX".

Then we need to find out what addresses are located on the same network using ping command.

```
echo 192.168.3.{1..254} | xargs -P255 -n1 ping -s1 -c1 -W1 | grep ttl

9 bytes from 192.168.3.1: icmp_seq=1 ttl=64
9 bytes from 192.168.3.28: icmp_seq=1 ttl=64
9 bytes from 192.168.3.98: icmp_seq=1 ttl=64
9 bytes from 192.168.3.5: icmp_seq=1 ttl=64
9 bytes from 192.168.3.30: icmp_seq=1 ttl=64
```

It seems 192.168.3.98 or 192.168.3.30 might be DE10-Nano's network address. You can try ssh with these addresses.  
Since DHCP often gives larger numbers to devices connected later, it is better to start with an address such as 192.168.3.98, which is the largest number.

**Note: If you cannot find another network address, make sure ethernet cable is firmly connected to DE10-Nano and try the command above several times.**

```
ssh root@192.168.3.98
root@192.168.3.98's password: de10nano
```

### Using GUI

GUI is the easiest approach in our opinion.
If you connect DE10-Nano to a display with HDMI cable, you'd find a GUI like this.

![DE10-Nano-LXDE-image](de10-nano-linux-c.md.pic/LXDE-Desktop.png)

If you'd like to connect DE10-Nano with your USB devices, you need to get USB Micro-A to USB Type-A adopter because  DE10-Nano only support **USB Micro-A port**. You have to get USB hub for connecting multiple USB devices to use GUI.

**Note: USB Micro-A cannot be substituted by USB Micro-B and Micro-B is often suggested when searching in web. Please be careful and do NOT make a mistake.**

<a name="InstallGreengrass"></a>
# Step 3: Install Greengrass Core

## 3.1 Install necessary packege and check greengrass dependency

###   Setup user and group

Open DE10-Nano terminal via GUI or SSH or serial console.
Create new user and group.

```
sudo adduser --system ggc_user
sudo addgroup --system ggc_group
```     

###   Install Java 8 runtime  
    
**This is needed if you use stream manager.**  
Greengrass Core Software 1.10 needs this Java 8 runtime to be installed in advance.

```
apt update
apt install openjdk-8-jdk
```

###  Use greengrass dependency chcker

```
mkdir -p /root/Downloads/greengrass-dependency-checker-GGCv1.10.x
cd /root/Downloads/greengrass-dependency-checker-GGCv1.10.x
wget https://github.com/aws-samples/aws-greengrass-samples/raw/master/greengrass-dependency-checker-GGCv1.10.x.zip
unzip greengrass-dependency-checker-GGCv1.10.x.zip
cd greengrass-dependency-checker-GGCv1.10.x
modprobe configs
sudo ./check_ggc_dependencies | more
```

Please check the output of shell script and make sure that your device is PASS.

## 3.2 Install Greengrass Core Software

### Get Greengrass Core security resources

Please follow the instruction [here](https://docs.aws.amazon.com/greengrass/latest/developerguide/gg-config.html) and download the serucity resources i.e. *hash*-setup.tar.gz.

### Install Greengrass Core software

Download Greengrass Core software [here](https://docs.aws.amazon.com/greengrass/latest/developerguide/what-is-gg.html#gg-core-download-tab).
The newest Greengrass Core software v1.10.0 is chosen here. It is recommended to have the newest version of core software. Please remember to change the dependency checker based on the version chosen.  

**There's no greengrass core for ARM32v7hf Ubuntu 16.04, thus we use the core for ARMv7l Raspbian.**  
As Raspbian and Ubuntu derive from Debian Linux, you can use the core for Raspbian as alternative.

![GreengrassCoreSoftware](de10-nano-linux-c.md.pic/GreengrassCoreSoftware.png)

### Copy Greengrass Core software and Greengrass security resources

Please search for the IP address of DE10-Nano first.
Open DE10-Nano terminal via GUI or SSH or serial console.

```
ip addr

2: eth0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc mq state UP group default qlen 1000
    link/ether da:c2:0c:13:e2:52 brd ff:ff:ff:ff:ff:ff
    inet 192.168.100.129/24 brd 192.168.100.255 scope global dynamic eth0
       valid_lft 604574sec preferred_lft 604574sec
    inet6 2001:268:c07f:1152:d89b:3b20:1cec:4/128 scope global dynamic 
       valid_lft 6975sec preferred_lft 3375sec
    inet6 2001:268:c07f:1152:a4e1:3f:c3e5:2724/64 scope global temporary dynamic
```

DE10-Nano has 192.168.100.129, as shown above.
Put your PC where greengrass core and security resouces are downloaded under the same network and execute scp to send them over.
```
cd *path-to-downloaded-files*
scp greengrass-linux-armv7l-1.10.0.tar.gz root@192.168.100.129:/root/Downloads
scp hash-setup.tar.gz root@192.168.100.129:/root/Downloads
```

### Extract Greengrass Core software and Greengrass security resources

In DE10-Nano terminal, extract the files.

```
cd /root/Downloads
tar -xvzf greengrass-linux-armv7l-1.10.0.tar.gz -C /
tar -xvzf hash-setup.tar.gz -C /greengrass
```

### Copy Root CA from AWS

```
cd /greengrass/certs/
sudo wget -O root.ca.pem https://www.amazontrust.com/repository/AmazonRootCA1.pem
```

Please make sure root.ca.pem has contents.

```
cat root.ca.pem
```

If you cannot see anything in root.ca.pem, please check if DE10-Nano can connect to the internet. If it doesn't work, you should doupt DNS setting or network setting.

```
ping google.com
```

### Run Greengrass Core

```
cd /greengrass/ggc/core
sudo ./greengrassd start
```

If you cannot run greengrassd, please refer to [this document](https://docs.aws.amazon.com/greengrass/latest/developerguide/gg-troubleshooting.html).

<a name="NextSteps"></a>
# Next Step

If you can run greengrassd, you can follow other modules published by AWS on their site.

Next you may follow [here](https://docs.aws.amazon.com/greengrass/latest/developerguide/module3-I.html) to run Lambda Functions on the core.

It is also better to make greengrassd as one of systemd services. You can follow it [here](https://docs.aws.amazon.com/greengrass/latest/developerguide/gg-core.html#start-on-boot).

<a name="References"></a>
# References

-   [Terasic Web Site for DE10-Nano Development Kit](de10-nano.terasic.com/cd) 
-   [Intel Web Site for DE10-Nano Development Kit](https://software.intel.com/en-us/iot/hardware/fpga/de10-nano)

[Bootable-image-link]: http://download.terasic.com/downloads/cd-rom/de10-nano/DE10-Nano-Cloud-Native.zip
[terasic-getting-started-guide]: https://www.terasic.com.tw/attachment/archive/1046/Getting_Started_Guide.pdf